Package: mwcsr
Title: Solvers for Maximum Weight Connected Subgraph Problem and Its
        Variants
Version: 0.1.11
Authors@R: c(person("Alexander", "Loboda", email = "aleks.loboda@gmail.com", role = c("aut", "cre")),
    person("Nikolay", "Poperechnyi", email = "n.poperechnyi@gmail.com", role = "aut"),
    person("Eduardo", "Alvarez-Miranda", email = "ealvarez@utalca.cl", role = "aut"), 
    person("Markus", "Sinnl", email = "markus.sinnl@univie.ac.at", role = "aut"),
    person("Alexey", "Sergushichev", email = "alsergbox@gmail.com", role = "aut"),
    person("Paul", "Hosler Jr.", role = "cph", comment = "Bundled JOpt Simple package"),
    person("www.hamcrest.org", role = "cph", comment = "Bundled hamcrest package"),
    person("Barak Naveh and Contributors", role = "cph", comment = "Bundled JGraphT package"),
    person("The Apache Software Foundation", role = "cph", comment = "Bundled Apache Commons Math package"))
Description: Algorithms for solving various Maximum
    Weight Connected Subgraph Problems, including variants with
    budget constraints, cardinality constraints, weighted edges and signals.
    The package represents an R interface to high-efficient solvers based on
    relax-and-cut approach (Álvarez-Miranda E., Sinnl M. (2017) <doi:10.1016/j.cor.2017.05.015>)
    mixed-integer programming (Loboda A., Artyomov M., and Sergushichev A. (2016) <doi:10.1007/978-3-319-43681-4_17>)
    and simulated annealing.
Depends: R (>= 3.5)
Imports: methods, igraph, Rcpp
Suggests: knitr, rmarkdown, mathjaxr, testthat, BioNet, roxygen2, DLBCL
SystemRequirements: Java (>=8)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr, rmarkdown
URL: https://github.com/ctlab/mwcsr
BugReports: https://github.com/ctlab/mwcsr/issues
LinkingTo: Rcpp
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-11 18:48:57 UTC; root
Author: Alexander Loboda [aut, cre],
  Nikolay Poperechnyi [aut],
  Eduardo Alvarez-Miranda [aut],
  Markus Sinnl [aut],
  Alexey Sergushichev [aut],
  Paul Hosler Jr. [cph] (Bundled JOpt Simple package),
  www.hamcrest.org [cph] (Bundled hamcrest package),
  Barak Naveh and Contributors [cph] (Bundled JGraphT package),
  The Apache Software Foundation [cph] (Bundled Apache Commons Math
    package)
Maintainer: Alexander Loboda <aleks.loboda@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 20:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-11 20:32:22 UTC; unix
Archs: mwcsr.so.dSYM

Package: mwcsr
Title: Solvers for Maximum Weight Connected Subgraph Problem and Its
        Variants
Version: 0.1.11
Authors@R: c(person("Alexander", "Loboda", email = "aleks.loboda@gmail.com", role = c("aut", "cre")),
    person("Nikolay", "Poperechnyi", email = "n.poperechnyi@gmail.com", role = "aut"),
    person("Eduardo", "Alvarez-Miranda", email = "ealvarez@utalca.cl", role = "aut"), 
    person("Markus", "Sinnl", email = "markus.sinnl@univie.ac.at", role = "aut"),
    person("Alexey", "Sergushichev", email = "alsergbox@gmail.com", role = "aut"),
    person("Paul", "Hosler Jr.", role = "cph", comment = "Bundled JOpt Simple package"),
    person("www.hamcrest.org", role = "cph", comment = "Bundled hamcrest package"),
    person("Barak Naveh and Contributors", role = "cph", comment = "Bundled JGraphT package"),
    person("The Apache Software Foundation", role = "cph", comment = "Bundled Apache Commons Math package"))
Description: Algorithms for solving various Maximum
    Weight Connected Subgraph Problems, including variants with
    budget constraints, cardinality constraints, weighted edges and signals.
    The package represents an R interface to high-efficient solvers based on
    relax-and-cut approach (Álvarez-Miranda E., Sinnl M. (2017) <doi:10.1016/j.cor.2017.05.015>)
    mixed-integer programming (Loboda A., Artyomov M., and Sergushichev A. (2016) <doi:10.1007/978-3-319-43681-4_17>)
    and simulated annealing.
Depends: R (>= 3.5)
Imports: methods, igraph, Rcpp
Suggests: knitr, rmarkdown, mathjaxr, testthat, BioNet, roxygen2, DLBCL
SystemRequirements: Java (>=8)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr, rmarkdown
URL: https://github.com/ctlab/mwcsr
BugReports: https://github.com/ctlab/mwcsr/issues
LinkingTo: Rcpp
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-11 18:48:57 UTC; root
Author: Alexander Loboda [aut, cre],
  Nikolay Poperechnyi [aut],
  Eduardo Alvarez-Miranda [aut],
  Markus Sinnl [aut],
  Alexey Sergushichev [aut],
  Paul Hosler Jr. [cph] (Bundled JOpt Simple package),
  www.hamcrest.org [cph] (Bundled hamcrest package),
  Barak Naveh and Contributors [cph] (Bundled JGraphT package),
  The Apache Software Foundation [cph] (Bundled Apache Commons Math
    package)
Maintainer: Alexander Loboda <aleks.loboda@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 20:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-11 20:15:06 UTC; unix
Archs: mwcsr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("[",mwcs_solver)
S3method("[<-",mwcs_solver)
S3method(check_parameter,char_parameter)
S3method(check_parameter,default)
S3method(check_parameter,file_parameter)
S3method(check_parameter,float_parameter)
S3method(check_parameter,integer_parameter)
S3method(check_parameter,logical_parameter)
S3method(check_parameter,mc_parameter)
S3method(parameters,default)
S3method(parameters,rmwcs_solver)
S3method(parameters,rnc_solver)
S3method(parameters,scipjack_solver)
S3method(parameters,simulated_annealing_solver)
S3method(parameters,virgo_solver)
S3method(print,mwcs_solver)
S3method(print,mwcs_solver_params)
S3method(solve_mwcsp,rmwcs_solver)
S3method(solve_mwcsp,rnc_solver)
S3method(solve_mwcsp,scipjack_solver)
S3method(solve_mwcsp,simulated_annealing_solver)
S3method(solve_mwcsp,virgo_solver)
export("timelimit<-")
export(annealing_solver)
export(get_instance_type)
export(get_weight)
export(normalize_sgmwcs_instance)
export(parameters)
export(rmwcs_solver)
export(rnc_solver)
export(scipjack_solver)
export(set_parameters)
export(solve_mwcsp)
export(virgo_solver)
import(igraph)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(stats,setNames)
useDynLib(mwcsr)

parameter <- function(name,
                      type = c("integer", "float", "logical", "mc", "file", "char"),
                      is_null_possible = FALSE,
                      positive = FALSE,
                      nonnegative = FALSE,
                      mc = NULL) {
    structure(list(name = name,
                   type = type,
                   is_null_possible = is_null_possible,
                   positive = positive,
                   nonnegative = nonnegative,
                   mc = mc),
              class = c(paste0(type, "_parameter"), parameter_class))
}

params <- function(...) {
    l <- list(...)
    names(l) <- lapply(l, function(x) x$name)
    structure(l, class = "mwcs_solver_params")
}

#' @export
print.mwcs_solver_params <- function (x, ...) {
    df <- data.frame(name = sapply(x, function(x) x$name))
    df[["type"]] <- sapply(x, function(x) x$type)
    df[["value"]] <- sapply(x, function(x) {
        if (x$type == "mc") {
            quote <- function(x) paste0("\"", x, "\"")
            paste0("{", paste0(sapply(x$mc, quote), collapse = ", "), "}")
        } else if (x$positive) {
            "positive"
        } else if (x$nonnegative) {
            "non-negative"
        } else {
            ""
        }
    })
    print(df, row.names = FALSE, right = FALSE)
}

check_parameter_class <- function(parameter) {
    if (!inherits(parameter, parameter_class)) {
        stop("Invalid argument")
    }
}

check_parameter <- function(param, value) {
    if (is.null(value)) {
        if (param$is_null_possible) {
            return()
        } else {
            stop(paste("Parameter", param$name, "mustn't be null"))
        }
    }
    if (length(value) == 0) {
        stop(paste("Parameter", param$name, "mustn't be a zero length vector"))
    }
    UseMethod("check_parameter")
}

#' @export
check_parameter.integer_parameter <- function (param, value) {
    value <- as.integer(value)
    if (is.na(value)) {
        stop(paste("Parameter", param$name, "mustn't be NA"))
    }
    if (param$positive) {
        if (value <= 0L) {
            stop(paste("Parameter", param$name, "must be positive"))
        }
    }
    if (param$nonnegative) {
        if (value < 0L) {
            stop(paste("Parameter", param$name, "must be non-negative"))
        }
    }
    value
}

#' @export
check_parameter.char_parameter <- function(param, value) {
    value
}

#' @export
check_parameter.float_parameter <- function (param, value) {
    value <- as.numeric(value)
    if (is.na(value)) {
        stop(paste("Parameter", param$name, "mustn't be NA"))
    }
    if (param$positive) {
        if (value <= 0) {
            stop(paste("Parameter", param$name, "must be positive"))
        }
    }
    if (param$nonnegative) {
        if (value < 0) {
            stop(paste("Parameter", param$name, "must be non-negative"))
        }
    }
    value
}

#' @export
check_parameter.logical_parameter <- function (param, value) {
    value <- as.logical(value)
    if (is.na(value)) {
        stop(paste("Parameter", param$name, "mustn't be NA"))
    }
    value
}

#' @export
check_parameter.mc_parameter <- function (param, value) {
    match.arg(value, param$mc)
}

#' @export
check_parameter.file_parameter <- function(param, value) {
    file <- normalizePath(value)
    if (!file.exists(file)) {
        stop(paste("File", file, "does not exist"))
    }
    file
}

#' @export
check_parameter.default <- function(param, value) {
    stop("Invalid state")
}

rnc_class <- "rnc_solver"

check_rnc_solver <- function(solver) {
    if (!inherits(solver, rnc_class)) {
        stop("Invalid relax&cut sgmwcs solver provided.")
    }
}

#' @export
parameters.rnc_solver <- function(solver) {
    params(parameter("max_iterations", type = "integer", positive = TRUE),
           parameter("beta_iterations", type = "integer", positive = TRUE),
           parameter("sep_iterations", type = "integer", positive = TRUE),
           parameter("heur_iterations", type = "integer", positive = TRUE),
           parameter("max_age", type = "integer", positive = TRUE),
           parameter("verbose", type = "logical"))
}

#' Construct relax-and-cut SGMWCS solver
#'
#' The solver is based on the same approach as rmwcs solver. Modifications to
#' the original scheme are introduced to tackle problems arising with introduction
#' of edge weights and signals. It is recommended to use rmwcs solver to solve MWCS
#' problems, while due to differences in primal heuristic it may be a good pratice
#' to run both solvers on the same problem.
#' @inheritParams rmwcs_solver
#' @return An object of class `mwcs_solver`
#' @seealso [rmwcs_solver]
#' @export
rnc_solver <- function(max_iterations = 1000L,
                       beta_iterations = 50L,
                       heur_iterations = 10L,
                       sep_iterations = 10L,
                       verbose = FALSE) {
    solver_ctor(c(rnc_class, mwcs_solver_class))
}

#' @rdname solve_mwcsp
#' @order 4
#' @export
solve_mwcsp.rnc_solver <- function(solver, instance, ...) {
    check_rnc_solver(solver)
    inst_type <- get_instance_type(instance)$type
    if (!(inst_type %in% c("MWCS", "GMWCS", "SGMWCS"))) {
        stop("Rnc solver supports only MWCS, GMWCS, SGMWCS problem instances")
    }

    signal_instance <- to_signal_instance(instance)

    inst_rep <- instance_from_graph(signal_instance)
    inst_rep[["vertex_signals"]] <- match(igraph::V(signal_instance)$signal, names(signal_instance$signals)) - 1
    inst_rep[["edge_signals"]] <- match(igraph::E(signal_instance)$signal, names(signal_instance$signals)) - 1
    inst_rep[["signal_weights"]] <- signal_instance$signals

    res <- sgmwcs_solve(inst_rep, solver)

    if (length(res$edges) == 0) {
        g <- igraph::induced_subgraph(instance, vids = res$vertices)
    } else {
        g <- igraph::subgraph_from_edges(instance, eids = res$edges)
    }

    weight <- get_weight(g)

    stopifnot(abs(weight - res$lb) < EPS)

    solution(g, weight, solved_to_optimality = abs(res$lb - res$ub) < EPS,
             upper_bound = res$ub)
}

scipjack_class = "scipjack_solver"

#' @export
parameters.scipjack_solver <- function(solver) {
    params(
        parameter("scipstp_bin", type="file", is_null_possible=FALSE),
        parameter("config_file", type="file", is_null_possible=TRUE))
}

write_stp <- function(g, stp_file) {
    edges <- igraph::as_edgelist(g, names = FALSE)
    nodes <- cbind("T", seq_along(V(g)), format(V(g)$weight, scientific=F, trim=T))
    n <- length(V(g))
    append <- function(s) {
        write(s, file=stp_file, append=TRUE)
    }
    write("33D32945 STP File, STP Format Version 1.0\n", file=stp_file,
         append=FALSE)

    append("SECTION Comments ")
    append("Problem \"Maximum Node Weight Connected Subgraph\"")
    append("END\n\n")

    append("SECTION Graph")
    append(paste0("Nodes ", n))
    append(paste0("Edges ", length(edges)/2))
    utils::write.table(cbind("E", edges), file=stp_file, quote=FALSE, sep = " ",
            col.names=FALSE, row.names=FALSE, append=TRUE)
    append("END\n\nSECTION Terminals")
    append(paste0("Terminals ", n))
    utils::write.table(nodes, file=stp_file, quote = FALSE, sep = " ",
            col.names=FALSE, row.names=FALSE, append=TRUE)
    append("END\n\nEOF")
}

append_output_file <- function(config_path, output_file) {
    write(paste0("stp/logfile=\"", output_file, "\""), file=config_path, append=TRUE)
}

find_output <- function(config_path) {
    lines <- readLines(config_path)
    idx <- grep('stp/logfile=*', lines)
    if (length(idx) == 0)
        path <- NULL
    else {
        path <- strsplit(lines[idx], '=')[[1]][2]
        path <- gsub("^\\s+|\\s+$", "", path)
    }
    path
}

run_scip <- function(solver, instance) {
    graph_dir <- tempfile("graph")
    dir.create(graph_dir, showWarnings=FALSE)
    default_output_file <- file.path(graph_dir, "out.stp")

    graph_file <- file.path(graph_dir, "in.stp")
    write_stp(instance, graph_file)

    config_path <- solver$config_file
    config_copy <- file.path(graph_dir, 'scip_config.s')
    if (!is.null(config_path)) {
        file.copy(config_path, config_copy)
    } else {
        file.create(config_copy)
    }

    output_file <- find_output(config_copy)
    if (is.null(output_file)) {
        append_output_file(config_copy, default_output_file)
        output_file <- default_output_file
    }

    system2(solver$scipstp_bin, c("-f", graph_file, "-s", config_copy), )

    lines <- readLines(output_file)
    nodes <- lines[grepl("^V \\d+", lines)]
    nodes <- as.integer(unlist(sub("V (\\d+)", '\\1', nodes)))

    induced_subgraph(instance, nodes)
}

run_scip_solver <- function(solver, instance) {
    V(instance)$index <- seq_len(vcount(instance))
    E(instance)$index <- seq_len(ecount(instance))
    mwcs <- run_scip(solver, instance)
    if (length(E(mwcs)) > 0) {
        ret <- igraph::subgraph.edges(instance, E(mwcs)$index)
    } else {
        ret <- igraph::induced_subgraph(instance, V(mwcs)$index)
    }
    solution(ret, get_weight(ret), TRUE, stats=NULL)
}

#' Construct a SCIP-jack solver
#'
#' This solver requires the STP extension of the SCIP-jack solver.
#' To use this class you first need to download and build `SCIP-jack` and
#' `SCIPSTP` application.
#'
#' You can access solver directly using `run_scip` function. See example.
#' @param scipstp_bin path to `scipstp binary`.
#' @param config_file scipstp-formatted file. Parameter names and values follow
#'        the SCIP configuration format.
#'
#' @export
#' @references Rehfeldt D., Koch T. (2019)
#' "Combining NP-Hard Reduction Techniques and Strong Heuristics in an Exact Algorithm for the Maximum-Weight Connected Subgraph Problem."
#' \doi{10.1137/17M1145963}
#' @examples
#' \dontrun{
#' data("bionet_example")
#' scip <- scipjack_solver(scipstp_bin='/path/to/scipoptsuite/build/bin/applications/scipstp')
#' sol <- solve_mwcsp(scip, bionet_example)
#' }

scipjack_solver <- function(scipstp_bin,
                            config_file=NULL) {
    solver_ctor((c(scipjack_class, mwcs_solver_class)))
}

#' @rdname solve_mwcsp
#' @order 5
#' @export
solve_mwcsp.scipjack_solver <- function(solver, instance, ...) {
    inst_type <- get_instance_type(instance)
    if (inst_type$type == "MWCS" && inst_type$valid) {
        return(run_scip_solver(solver, instance))
    } else {
        stop("Not a valid MWCS instance")
    }
}

scip_bin <- Sys.getenv("SCIPSTP_BIN")

test_that("SCIP solver does not crash on GAM instances", {
    if (!file.exists(scip_bin)) {
        skip("SCIP is not available")
    }
    solver <- scipjack_solver(scip_bin)
    for (graph in GAM) {
        solution <- solve_mwcsp(solver, graph)
        expect_gte(length(V(solution$graph)), 0)
    }
})

test_that("SCIP solver supports config file", {
    if (!file.exists(scip_bin)) {
        skip("SCIP is not available")
    }
    scip_path <- system.file('ext', 'scip_config.s')
    solver <- scipjack_solver(scip_bin, scip_path)
    for (graph in GAM) {
        solution <- solve_mwcsp(solver, graph)
        expect_gte(length(V(solution$graph)), 0)
    }
})

rmwcs_class <- "rmwcs_solver"

sep_methods <- c("strong", "fast")
subgradients <- c("classic", "average", "cft")

check_rmwcs_solver <- function(solver) {
    if (!inherits(solver, rmwcs_class)) {
        stop("Function called with an invalid rmwcs solver instance")
    }
}

#' @export
parameters.rmwcs_solver <- function(solver) {
    params(parameter("timelimit", type = "integer", positive =  TRUE,
                   is_null_possible = TRUE),
         parameter("max_iterations", type = "integer", positive = TRUE,
                   is_null_possible = TRUE),
         parameter("max_age", type = "integer", positive = TRUE),
         parameter("beta_iterations", type = "integer", positive = TRUE),
         parameter("separation", type = "mc", mc = sep_methods),
         parameter("sep_iterations", type = "integer", positive = TRUE),
         parameter("start_constraints", type = "logical"),
         parameter("pegging", type = "logical"),
         parameter("sep_iter_freeze", type = "integer", positive = TRUE),
         parameter("heur_iterations", type = "integer", positive = TRUE),
         parameter("subgradient", type = "mc", mc = subgradients),
         parameter("beta", type = "float"),
         parameter("verbose", type = "logical"))
}

#' Generate a rmwcs solver
#'
#' The method is based on relax-and-cut approach and allows to solve
#' Maximum Weight Subgraph Probleam and its budget and cardinality variants.
#' By constructing lagrangian
#' relaxation of MWCS problem necessary graph connectivity constraints are introduced
#' in the objective function giving upper bound on the weight of the optimal
#' solution. On the other side, primal heuristic uses individul contribution
#' of the variables to lagrangian relaxation to find possible solution of the initial
#' problem. The relaxation is then optimized by using iterative subgradient method.
#'
#' One iteration of algorithm includes solving lagrangian relaxation and updating
#' lagrange multipliers. It may also contain cuts (or connectivity constraints) separation process, run of
#' heuristic method, variable fixing routine. The initial step size for
#' subgradient method can be passed as `beta` argument. If there is no improvement in
#' upper bound in consequtive `beta_iterations` iterations the step size is
#' halved. There are three possible strategies for updating multipliers. See the references
#' section for the article where differences are discussed.
#'
#' Some initial cuts are added at the start of the algorithm if `start_constraints`
#' is set to \code{TRUE}. Other constraints are separated on the fly and are
#' unaffected in the next `sep_iter_freeze` iterations of the subgradient mehod.
#' Then the corresponding lagrange mutipliers are updated from iteration to iteration.
#' Aging procedure for cuts is incorporated in the algorithm meaning constraint multipliers
#' are updated for non-violated cuts for up to `max_age` iterations from
#' the point where a cut was violated last time. There are two separation methods
#' implemented: fast and strong, where tha latter is supposed to minimize number of
#' variables used in generated constraint while in the former case there is no need to explore
#' whole graph to construct a constraint.
#'
#' A variant of MST approximation of PCSTP is used as Primal Heuristic.
#' See references for more details.
#'
#' The frequences
#' of running separation process and heuristic are specified in
#' `sep_iterations` and `heur_iterations`.
#'
#' @param timelimit Timelimit in seconds
#' @param max_iterations Maximum number of iterations
#' @param start_constraints Whether to add flow-conservation/degree constraints at start
#' @param separation Separation: "strong" or "fast"
#' @param sep_iterations Frequency of separating cuts (in iterations)
#' @param beta_iterations Number of nonimproving iterations until beta is halved
#' @param pegging variable fixing
#' @param max_age number of iterations in aging procedure for non-violated  cuts
#' @param sep_iter_freeze Number of iterations when a newly separated cut is anaffected by subgradient algorithm.
#' @param heur_iterations Frequency of calling heuristic method (in iterations)
#' @param subgradient Subgradient: "classic", "average", "cft"
#' @param beta Initial step size of subgradient algorithm
#' @param verbose Should the solving progress and stats be printed?
#' @return An object of class `mwcs_solver`.
#' @references Álvarez-Miranda E., Sinnl M. (2017)
#' "A Relax-and-Cut framework for large-scale maximum weight connected subgraph problems"
#' \doi{10.1016/j.cor.2017.05.015}
#' @export
#' @import igraph
rmwcs_solver <- function(timelimit = 1800L,
                  max_iterations = 1000L,
                  beta_iterations = 5L,
                  separation = "strong",
                  start_constraints = TRUE,
                  pegging = TRUE,
                  max_age = 10,
                  sep_iterations= 10L,
                  sep_iter_freeze = 50L,
                  heur_iterations = 10L,
                  subgradient = "classic",
                  beta = 2.0,
                  verbose = FALSE) {
    solver_ctor(c(rmwcs_class, mwcs_solver_class))
}

#' @rdname solve_mwcsp
#' @param max_cardinality integer maximum number of vertices in solution.
#' @param budget numeric maximum budget of solution.
#' @order 3
#' @export
solve_mwcsp.rmwcs_solver <- function(solver, instance, max_cardinality = NULL,
                                     budget = NULL, ...) {
    check_rmwcs_solver(solver)
    inst_type <- get_instance_type(instance)$type
    if (!(inst_type %in% c("MWCS", "Budget MWCS"))) {
        stop("Rmwcs solver supports only MWCS and Budget MWCS problem instances")
    }

    if (!is.null(max_cardinality) && !is.null(budget)) {
        stop("One of the arguments 'max_cardinality' and 'budget' must be NULL")
    }

    inst_type <- get_instance_type(instance)
    if (!inst_type$type %in% c("MWCS", "Budget MWCS") && !inst_type$valid) {
        stop("Instance is not a valid MWCS nor Budget MWCS instance")
    }

    instance <- igraph::simplify(instance)

    solver$separation <- pmatch(solver$separation, sep_methods) - 1
    solver$subgradient <- pmatch(solver$subgradient, subgradients) - 1

    instance_rep <- instance_from_graph(instance)
    instance_rep$vertex_weights <- V(instance)$weight

    if (!is.null(budget)) {
        instance_rep$budget_cost <- V(instance)$budget_cost
        budget <- as.numeric(budget)
        instance_rep$budget <- budget
    }

    if (!is.null(max_cardinality)) {
        max_cardinality <- as.integer(max_cardinality)
        stopifnot(max_cardinality > 0)
        instance_rep$cardinality <- max_cardinality
    }

    vs <- rmwcs_solve(instance_rep, solver)

    subgraph <- igraph::induced_subgraph(instance, vs$graph)
    weight <- sum(instance_rep$vertex_weights[vs$graph])
    opt <- isTRUE(all.equal(weight, vs$ub))
    solution(subgraph, weight, opt, upper_bound = vs$ub, incumbent = vs$lb,
             solved_to_optimality = abs(vs$lb - vs$ub) < EPS)
}

data("gam_example")
set.seed(42L)

test_that("rmwcs solver can be constructed with specific parameters", {
    solver <- rmwcs_solver()
    expect_equal(solver$separation, "strong")
    timelimit(solver) <- 20
    expect_equal(solver$timelimit, 20)
})


test_that("one vertex best solution is returned", {
    solver <- rmwcs_solver()
    g <- igraph::make_graph(c("A", "B"), directed = FALSE)
    V(g)$weight <- c(5, -2)
    solution <- solve_mwcsp(solver, g)
    expect_equal(solution$weight, 5)
})

test_that("rmwcs solver works on specific test", {
    solver <- rmwcs_solver()
    g <- make_ring(5) %>% set_vertex_attr("weight", value = 2:-2)
    s <- solve_mwcsp(solver, g)
    expect_equal(s$weight, 3)
    expect_true(s$solved_to_optimality)
})

test_that("rmwcs solver doesn't crash on simple graphs", {
    solver <- rmwcs_solver()
    size <- 10
    card <- 3

    test_graph <- function(make) {
        scores <- runif(size) - 0.5;
        g <- make(size) %>% set_vertex_attr("weight", value = scores)
        solve_mwcsp(solver, g)

        solution <- solve_mwcsp(solver, g, max_cardinality = card)
        expect_lte(length(V(solution$graph)), 3)

        g <- set_vertex_attr(g, name = "budget_cost", value = runif(size))
        V(g)$weight <- 1
        solution <- solve_mwcsp(solver, g, budget = card)

        expect_lte(sum(V(solution$graph)$budget_cost), card)
    }

    test_graph(make_ring)
    test_graph(function(x) make_star(x, mode = "undirected"))
    test_graph(make_full_graph)
})

test_that("rmwcs solver builds connected solutions on a GAM instance", {
    solver <- rmwcs_solver()
    sol <- solve_mwcsp(solver, gam_example)
    expect_true(is_connected(sol$graph))
})

test_that("rmwcs solver works with empty solutions", {
    solver <- rmwcs_solver()
    g <- make_ring(10)
    V(g)$weight <- -1
    sol <- solve_mwcsp(solver, g)
    expect_true(vcount(sol$graph) == 0)
})

solution <- function(graph, weight, solved_to_optimality = FALSE, ...) {
    stopifnot((igraph::vcount(graph) == 0) || igraph::is_connected(graph))
    obj <- c(list(graph = graph, weight = weight,
                  solved_to_optimality = solved_to_optimality), list(...))
    structure(obj, class = "mwcsp_solution")
}

#' Calculate weight of the solution. MWCS, GMWCS and SGMWCS instances are supported
#' @param solution Either `mwcsp_solution` or `igraph`` object representing the solution
#' @return Weight of the subgraph
#' @export
#' @importFrom methods is
get_weight <- function(solution) {
    if (is(solution, "mwcsp_solution")) {
        return(get_weight(solution$graph))
    }
    type <- get_instance_type(solution)$type
    if (type == 'SGMWCS') {
        active_signals <- unique(c(V(solution)$signal, E(solution)$signal))
        scores <- c(solution$signals)
        return (sum(scores[active_signals]))
    }
    else if (type == 'GMWCS') {
        return (sum(c(V(solution)$weight, E(solution)$weight)))
    } else if (type == 'MWCS') {
        return (sum(c(V(solution)$weight)))
    } else {
        stop('unexpected graph type. Expected one of: MWCS, GMWCS, SGMWCS')
    }
}

library(igraph)

test_that("one vertex best solution is returned", {
    solver <- rnc_solver(max_iterations = 100)
    g <- igraph::make_graph(c("A", "B"), directed = FALSE)
    V(g)$weight <- c(5, 5)
    E(g)$weight <- -7
    solution <- solve_mwcsp(solver, g)
    expect_equal(solution$weight, 5)
})

test_that("sgmwcs rnc solver works on specific test", {
    solver <- rnc_solver(max_iterations = 100)
    g <- igraph::make_ring(5)
    V(g)$weight <- 1:-3
    E(g)$weight <- 1
    solution <- solve_mwcsp(solver, g)
    expect_equal(solution$weight, 2)
    expect_equal(solution$solved_to_optimality, TRUE)
    expect_true(igraph::is_connected(solution$graph))
})

test_that("sgmwcs rnc solver does not crush on GAM instances", {
    solver <- rnc_solver(max_iterations = 100)
    solution <- solve_mwcsp(solver, gam_example)
    expect_gte(length(V(solution$graph)), 0)
    expect_true(igraph::is_connected(solution$graph))
})

test_that("rnc solver handles non-integer signal weights in single-vertex solution (#10)", {
    solver <- rnc_solver(max_iterations = 50)
    g <- igraph::make_graph(c(1,2, 3,4, 3,5, 1,6, 4,6) + 0L, n = 6, directed = FALSE)
    V(g)$signal <- c("s2","s2","s2","s2","s3","s2")
    E(g)$signal <- c("s1","s2","s3","s2","s2")
    g$signals <- c(s1 = -3.026, s2 = 2.193558, s3 = -4.921)
    solution <- solve_mwcsp(solver, g)
    expect_equal(solution$weight, 2.193558)
})

test_that("sgmwcs rnc solver gives good solution for a GAM instance", {
    rnc <- rnc_solver(max_iterations = 100)
    solution <- solve_mwcsp(rnc, gmwcs_example)
    expect_gte(get_weight(solution$graph), 200)
    expect_true(igraph::is_connected(solution$graph))
})

#' GAM instance for MWCS problem
#'
#' A dataset containing some real-world instances appeared in network
#' enrichment analysis tool Shiny GAM
#' (\doi{10.1093/nar/gkw266}).
#'
#' @format A vector of named vertex-weighted igraph instances
#' @source \url{http://dimacs11.zib.de/instances/MWCS-GAM.zip}
"gam_example"

#' Example MWCS instance
#'
#' Instance is based on `gatom` package.
"mwcs_example"

#' Example GMWCS instance
#'
#' Instance is based on `gatom` package.
"gmwcs_example"

#' Example SGMWCS instance
#'
#' Instance is based on `gatom` package.
"sgmwcs_example"


#' Example MWCS instance obtained from BioNet package tutorial
#'
"bionet_example"

#' Example of graph from which an SGMWCS instance can be obtained
#'
#' The graph is based on `gatom` package
"gatom_example"

#' Small example of MWCS instance for demonstration purposes.
"mwcs_small_instance"

#' Small example of GMWCS instance for demonstration purposes.
"gmwcs_small_instance"

#' Small example of SGMWCS instance for demonstration purposes.
"sgmwcs_small_instance"

virgo_java_class = "ru.itmo.ctlab.virgo.Main"
virgo_class = "virgo_solver"

#' @export
parameters.virgo_solver <- function(solver) {
    params(parameter("cplex_bin", type = "file", is_null_possible=TRUE),
         parameter("cplex_jar", type = "file", is_null_possible=TRUE),
         parameter("threads", type = "integer", positive = TRUE),
         parameter("timelimit", type = "integer", positive = TRUE,
                   is_null_possible = TRUE),
         parameter("penalty", type = "float", nonnegative = TRUE, is_null_possible=FALSE),
         parameter("memory", type = "char"),
         parameter("mst", type = "logical"),
         parameter("dryrun", type = "logical", is_null_possible=FALSE),
         parameter("jvmargs", type = "char", is_null_possible = TRUE),
         parameter("log", type = "integer", is_null_possible=TRUE))
}

init_solver <- function(solver) {
    solver_name <- "virgo-solver.jar"
    solver_jar <- system.file("java", solver_name, package="mwcsr")
    command <- c(paste0("-Xmx", solver$memory), solver$jvmargs)
    if (is.null(solver$cplex_bin) || is.null(solver$cplex_jar)) {
        command <- c(command, "-cp", solver_jar, virgo_java_class)
    } else {
        sep <- ifelse(.Platform$OS.type == "unix", ":", ";")
        command <- c(command, sprintf("-Djava.library.path=%s", shQuote(solver$cplex_bin)),
                    "-cp", shQuote(paste(solver_jar, solver$cplex_jar, sep=sep)),
                    virgo_java_class
                    )

    }

    solver$run_main <- function(cli_args, loglevel = 2) {
        exit_code <- NULL
        if (solver$dryrun) {
            cat(paste(c(shQuote("java"), command, cli_args), collapse=" "), "\n")
            stop("Dry run")
        }
        if (loglevel == 0) {
            exit_code <- system2("java", c(command, cli_args), stdout = FALSE)
        } else {
            exit_code <- system2("java", c(command, cli_args))
        }
        if (exit_code != 0) {
            stop("Failed to run java-based solver.")
        }
    }
    solver
}

find_cplex_jar <- function(cplex_dir) {
    files <- list.files(cplex_dir, pattern = "cplex.jar", recursive=T, full.names = TRUE)
    if (length(files) > 0) {
        return(files[1])
    } else {
        return(NULL)
    }
}

find_cplex_bin <- function(cplex_dir) {
    cplex_lib_pattern <- "(cplex\\d+\\.dll)|libcplex\\d+.(so|jnilib|dll)"
    files <- list.files(cplex_dir, pattern = cplex_lib_pattern, recursive=T, full.names = TRUE)
    if (length(files) > 0) {
        return(dirname(files[1]))
    } else {
        return(NULL)
    }
}


#' Construct a virgo solver
#'
#' This solver uses reformulation of MWCS problem in terms of mixed integer
#' programming. The later problem can be efficiently solved with
#' commercial optimization software. Exact version of solver uses CPLEX and requires
#' it to be installed. CPLEX 12.7.1 or higher is required.
#'
#' The solver currently does not support repeated negative signals, i.e. every
#' negative signal should be present only once among all edges and vertices.
#'
#' You can access solver directly using `run_main` function. See example.
#' @param cplex_dir a path to dir containing cplex_bin and cplex_jar,
#'        setting this to NULL sets `mst` param to `TRUE`
#' @param cplex_bin a path to cplex binary dir
#' @param cplex_jar a path to cplex jar file
#' @param threads number of threads for simultaneous computation
#' @param timelimit maximum number of seconds to solve the problem
#' @param memory maximum amount of memory(-Xmx flag)
#' @param log verbosity level
#' @param penalty additional edge penalty for graph edges
#' @param mst whether to use approximate MST solver, no CPLEX files required with this parameter
#'            is set to `TRUE`
#' @param dryrun if set to `TRUE` only prints the solver command, without actually running it
#' @param jvmargs character vector with additional arguments for Java Virtual Machine
#' @return An object of class `mwcs_solver`.
#' @references Loboda A., Artyomov M., and Sergushichev A. (2016)
#' "Solving generalized maximum-weight connected subgraph problem for network enrichment analysis"
#' \doi{10.1007/978-3-319-43681-4_17}
#' @export
#' @examples
#' data("sgmwcs_small_instance")
#' approx_vs <- virgo_solver(mst=TRUE, threads = 1)
#' approx_vs$run_main("-h")
#' sol <- solve_mwcsp(approx_vs, sgmwcs_small_instance)
#' \dontrun{
#' vs <- virgo_solver(cplex_dir='/path/to/cplex')
#' sol <- solve_mwcsp(vs, sgmwcs_example)
#' }
virgo_solver <- function (cplex_dir,
                          threads = parallel::detectCores(),
                          timelimit = NULL,
                          penalty = 0.0,
                          memory = "2G",
                          log = 0,
                          cplex_bin=NULL,
                          cplex_jar=NULL,
                          mst=FALSE,
                          dryrun=FALSE,
                          jvmargs=NULL
                          ) {

    if (missing(cplex_dir)) {
        if (!mst && (is.null(cplex_bin) || is.null(cplex_jar))) {
            stop("Either provide `cplex_dir` parameter or both `cplex_bin` and `cplex_jar`")
        }
        cplex_dir<-NULL
    } else {
        if (is.null(cplex_dir)) {
            mst <- TRUE
        }
    }


    if (is.null(cplex_jar) && !mst) {
        cplex_jar <- find_cplex_jar(cplex_dir)
        if (is.null(cplex_jar)) {
            stop(paste0("Could not find `cplex.jar` in ", cplex_dir))
        }
    }

    if (is.null(cplex_bin) && !mst) {
        cplex_bin <- find_cplex_bin(cplex_dir)
        if (is.null(cplex_bin)) {
            stop(paste0("Could not find libcplex files in ", cplex_dir))
        }
    } else if (!mst && !dir.exists(cplex_bin)) {
        # cplex_bin should be directory, not the lib-file itself
        cplex_bin <- dirname(cplex_bin)
    }

    rm(cplex_dir) # don't need this parameter anymore

    init_solver(solver_ctor(c(virgo_class,mwcs_solver_class)))
}

write_files <- function(g, nodes_file, edges_file, signals_file, signals) {
    # avoid printing scientific values
    scipen = getOption('scipen')
    options(scipen=max(scipen, 50))
    edges <- igraph::as_edgelist(g, names = FALSE)
    write_tbl <- function(x, file, rn) {
        utils::write.table(x, file = file, quote = FALSE, sep = "\t",
            row.names = rn, col.names = FALSE
        )
    }
    if (is.null(signals)) {
        node_weights <- V(g)$weight
        if ("weight" %in% edge_attr_names(g)) {
            edge_weights <- E(g)$weight
        } else {
            edge_weights <- rep(0, ecount(g))
        }

        edges <- cbind(edges, edge_weights)
        vertices <- cbind(seq_along(V(g)), node_weights)
    } else {
        node_signals <- igraph::vertex_attr(g, "signal")
        edge_signals <- igraph::edge_attr(g, "signal")
        edges <- cbind(edges, edge_signals)
        vertices <- cbind(seq_along(V(g)), node_signals)
        write_tbl(signals, signals_file, TRUE)
    }
    write_tbl(vertices, nodes_file, FALSE)
    write_tbl(edges, edges_file, FALSE)
    options(scipen=scipen)
}

cli_args <- function(solver, nodes.file, edges.file, signals.file = NULL, stats.file = NULL) {
    loglevel <- max(1, solver$log)
    params <- c("-n", nodes.file, "-e", edges.file, "-m", solver$threads, "-l", loglevel)
    if (!is.null(solver$timelimit)) {
        params <- c(params, "-t", solver$timelimit)
    }
    if (!is.null(signals.file)) {
        params <- c(params, "-s", signals.file)
        params <- c(params, "-type", "sgmwcs")
    } else {
        params <- c(params, "-type", "gmwcs")
    }
    if (!is.null(solver$penalty)) {
        params <- c(params, "-p", solver$penalty)
    }
    if (!is.null(stats.file)) {
        params <- c(params, "-f", stats.file)
    }
    if (solver$mst) {
        params <- c(params, "-mst")
    }
    params
}

run_solver <- function(solver, instance, sgmwcs, signals = NULL) {
    graph_dir <- tempfile("graph")
    dir.create(graph_dir, showWarnings=FALSE)
    edges.file <- file.path(graph_dir, "edges.txt")
    nodes.file <- file.path(graph_dir, "nodes.txt")
    signals.file <- if (sgmwcs) file.path(graph_dir, "signals.txt") else NULL
    stats.file <- file.path(graph_dir, "stats.tsv")

    write_files(instance, nodes.file, edges.file, signals.file, signals)
    args <- cli_args(solver, nodes.file, edges.file, signals.file, stats.file)
    solver$run_main(args, solver$log)

    colnames <- NULL
    if (sgmwcs) {
        colnames <- list(nodes = c("node"),
                         edges = c("from", "to", "signal"))
    } else {
        colnames <- list(nodes = c("node", "score"),
                         edges = c("from", "to", "score"))
    }

    nodes <- utils::read.table(paste0(nodes.file, ".out"), comment.char = "#",
                               col.names = colnames$nodes)
    edges <- utils::read.table(paste0(edges.file, ".out"), comment.char = "#",
                               col.names = colnames$edges)

    if (!sgmwcs) {
        nodes <- nodes[nodes[, 2] != "n/a", ]
        edges <- edges[edges[, 3] != "n/a", ]
    }

    stats <- utils::read.table(stats.file, header = T)
    mwcs <- (if (nrow(nodes) <= 1) {
        induced_subgraph(instance, as.integer(nodes[, 1]))
    } else {
        eids <- get_edge_ids(instance, c(t(edges[,1:2])))
        subgraph_from_edges(instance, eids)
    })
    list(mwcs=mwcs, stats=stats)
}

#' @rdname solve_mwcsp
#' @order 2
#' @export
#'
solve_mwcsp.virgo_solver <- function(solver, instance, ...) {
    inst_type <- get_instance_type(instance)
    if (inst_type$type == "SGMWCS" && inst_type$valid) {
        return(solve_sgmwcs(solver, instance, ...))
    } else if (inst_type$type %in% c("GMWCS", "MWCS") && inst_type$valid) {
        return(solve_gmwcs(solver, instance, ...))
    } else {
        msg <- "Not a valid MWCS, GMWCS or SGMWCS instance"
        if (inst_type$type != "unknown") {
            msg <- paste0(msg, sprintf("\nThe instance looks like %s", inst_type$type))
            if (!inst_type$valid) {
                msg <- paste0(msg, ", but there was an error:")
                msg <- paste0(c(msg, inst_type$errors), collapse = "\n")
            }
        }
        stop(msg)
    }
}

solve_sgmwcs <- function(solver, instance, ...) {
    signals <- instance$signals

    neg_signals <- names(which(signals < 0))
    if (any(table(c(V(instance)$signal, E(instance)$signal))[neg_signals] > 1, na.rm=TRUE)) {
        stop("Instances with repeated negative signals are not supported")
    }

    V(instance)$index <- seq_len(vcount(instance))
    E(instance)$index <- seq_len(ecount(instance))

    sol <- run_solver(solver, instance, sgmwcs = TRUE,
                      data.frame(score=instance$signals, row.names=names(instance$signals)))
    mwcs <- sol$mwcs
    sigs <- union(V(mwcs)$signal, E(mwcs)$signal)
    weight <- sum(signals[sigs])
    if (length(E(mwcs)) > 0) {
        ret <- igraph::subgraph_from_edges(instance, E(mwcs)$index)
    } else {
        ret <- igraph::induced_subgraph(instance, V(mwcs)$index)
    }
    solution(ret, weight, sol$stats$isOpt == 1, stats=sol$stats)
}

solve_gmwcs <- function(solver, instance, ...) {
    inst_type <- get_instance_type(instance)
    if (!inst_type$type %in% c("GMWCS", "MWCS") || !inst_type$valid) {
        stop("Not a valid GMWCS instance")
    }

    sol <- run_solver(solver, instance, sgmwcs = FALSE)
    mwcs <- sol$mwcs

    weight <- sum(V(mwcs)$weight, E(mwcs)$weight)
    solution(mwcs, weight, sol$stats$isOpt == 1, stats=sol$stats)
}

cplex_dir <- normalizePath(Sys.getenv("CPLEX_HOME"))

test_that("virgo_solver works on MWCS", {
    if (!file.exists(cplex_dir)) {
        skip("No CPLEX available")
    }
    solver <- virgo_solver(cplex_dir=cplex_dir)
    solution <- solve_mwcsp(solver, mwcs_small_instance)
    expect_equal(solution$weight, 3)
})


test_that("virgo_solver works on GMWCS", {
    if (!file.exists(cplex_dir)) {
        skip("No CPLEX available")
    }
    solver <- virgo_solver(cplex_dir=cplex_dir)
    solution <- solve_mwcsp(solver, gmwcs_small_instance)
    expect_equal(solution$weight, 11)
})

test_that("virgo_solver works on SGMWCS", {
    if (!file.exists(cplex_dir)) {
        skip("No CPLEX available")
    }
    solver <- virgo_solver(cplex_dir=cplex_dir)
    solution <- solve_mwcsp(solver, sgmwcs_small_instance)
    expect_equal(solution$weight, 51)
})


test_that("heuristic virgo_solver works on MWCS", {
    solver <- virgo_solver(cplex_dir=NULL)
    solution <- solve_mwcsp(solver, mwcs_small_instance)
    expect_equal(solution$weight, 3)
})


test_that("heuristic virgo_solver works on GMWCS", {
    solver <- virgo_solver(cplex_dir=NULL)
    solution <- solve_mwcsp(solver, gmwcs_small_instance)
    expect_gt(solution$weight, 0)
    expect_lte(solution$weight, 11)
})

test_that("heuristic virgo_solver works on SGMWCS", {
    solver <- virgo_solver(cplex_dir=NULL)
    solution <- solve_mwcsp(solver, sgmwcs_small_instance)
    expect_equal(solution$weight, 51)
})

test_that("virgo solver doesn't fail on empty output", {
    solver <- virgo_solver(cplex_dir = NULL)
    g <- igraph::make_full_graph(2)
    igraph::E(g)$signal <- "S1"
    igraph::V(g)$signal <- c("S2", "S3")
    g$signals <- stats::setNames(c(-1, -2, -1), c("S1", "S2", "S3"))
    expect_true(!is.null(solve_mwcsp(solver, g)))
})

test_that("virgo solver does not supported repeated negative signals", {
    sgmwcs_edges <- data.frame(from = c(1, 2, 2, 3, 4, 5, 1),
                               to = c(2, 3, 4, 4, 6, 6, 5),
                               signal = c("S6", "S2", "S7", "S2", "S8", "S9", "S10"))
    sgmwcs_instance <- igraph::graph_from_data_frame(sgmwcs_edges,
                                                     directed = FALSE)
    igraph::V(sgmwcs_instance)$signal <- c("S1", "S3", "S4", "S5", "S1", "S1")
    sgmwcs_instance$signals <- stats::setNames(c(7.0, -20.0, 40.0, 15.0, 8.0, 3.0, -7.0, -10.0,
                                                 -2.0, -15.3, 1.0),
                                               paste0("S", 1:11))

    solver <- virgo_solver(cplex_dir=NULL)
    expect_error(solution <- solve_mwcsp(solver, sgmwcs_instance))

})

test_that("heuristic virgo_solver works on SGMWCS", {
    solver <- virgo_solver(cplex_dir=NULL)
    si <- sgmwcs_small_instance
    si$signals <- c(si$signals, "neg"=-1)
    solution <- solve_mwcsp(solver, si)
    expect_true(!is.null(solution))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sa_solve <- function(instance, solver) {
    .Call('_mwcsr_sa_solve', PACKAGE = 'mwcsr', instance, solver)
}

sgmwcs_solve <- function(instance, solver_params) {
    .Call('_mwcsr_sgmwcs_solve', PACKAGE = 'mwcsr', instance, solver_params)
}

rmwcs_solve <- function(network, params) {
    .Call('_mwcsr_rmwcs_solve', PACKAGE = 'mwcsr', network, params)
}

sa_class <- "simulated_annealing_solver"
schedules <- c("fast", "boltzmann")

check_sa_solver <- function(solver) {
    if (!inherits(solver, sa_class)) {
        stop("Function called with an invalid SA solver instance")
    }
}

#' @export
parameters.simulated_annealing_solver <- function(solver) {
    params(parameter("schedule", type = "mc", mc = schedules),
        parameter("initial_temperature", type = "float", positive = TRUE),
        parameter("final_temperature", type = "float", positive = TRUE),
        parameter("verbose", type = "logical"))
}

#' Construct an annealing solver
#'
#' Simulated annealing is a heuristic method of solving optimization problems.
#' Typically, it allows to find some good solution in a short time. This
#' implementation doesn't compute any upper bound on solution, so there is no
#' guarantee of optimality of solution provided.
#'
#' Algorithm maintains connected subgraph staring with empty subgraph.
#' Each iteration one random action is considered. It may be a removal of a
#' vertex or an edge which does not separate graph or addition of an extra vertex or
#' an edge neighboring existing graph. If the subgraph is empty all vertices
#' are considered as candidates to form a subgaph. After candidate is chosen two
#' subgraph scores are considered: for a new subgraph and for an old one. Simulated
#' annealing operates with a notion of temperature. The candidate action is
#' accepted with probability: p(S'|S) = exp(-E / T), where E is weight difference
#' between subgraphs and T is current temperature.
#'
#' Temperature is calculated in each iteration. in `mwcsr` there are two
#' temperature schedules supported. So called Boltmann annealing uses the formula:
#' T(k) = T0 / (ln(1 + k)), while in case of fast annealing this one is used:
#' T(k) = T0 / k, where k is iteration number.
#'
#' To tune the algorithm it is useful to realize how typical changes in the goal
#' function for single actions are distributed. Calculating the acceptance probabilities
#' at initial temperature and final temperatures may help to choose schedule and
#' temperatures.
#'
#' @seealso [rnc_solver] will probably be a better choice with minimal tuning necessary
#' @param schedule boltzmann annealing or fast annealing
#' @param initial_temperature initial value for the temperature
#' @param final_temperature final value for the temperature
#' @param verbose whether be verbose or not
#' @return An object of class `mwcs_solver`
#' @export
annealing_solver <- function(schedule = c("fast", "boltzmann"),
                             initial_temperature = 1.0,
                             final_temperature = 1e-6,
                             verbose = FALSE) {
    solver_ctor(c(sa_class, mwcs_solver_class))
}

#' @rdname solve_mwcsp
#' @order 5
#' @param warm_start warm start solution, an object of the class mwcsp_solution.
#' @export
solve_mwcsp.simulated_annealing_solver <- function(solver, instance, warm_start, ...) {
    if (!inherits(solver, sa_class)) {
        stop("Not a simulated annealing solver")
    }

    if (!(get_instance_type(instance)$type %in% c("MWCS", "GMWCS", "SGMWCS"))) {
        stop("Annealing solver supports only MWCS, GMWCS, SGMWCS problem instances")
    }

    signal_instance <- to_signal_instance(instance)

    inst_rep <- instance_from_graph(signal_instance)
    inst_rep[["vertex_signals"]] <- match(igraph::V(signal_instance)$signal, names(signal_instance$signals)) - 1
    inst_rep[["edge_signals"]] <- match(igraph::E(signal_instance)$signal, names(signal_instance$signals)) - 1
    inst_rep[["signal_weights"]] <- signal_instance$signals

    if (!missing(warm_start)) {
        ws_sol <- warm_start$warm_start_solution
        inst_rep[["warm_start_vertices"]] <- ws_sol$vertices
        inst_rep[["warm_start_edges"]] <- ws_sol$edges
        inst_rep[["warm_start_weight"]] <- ws_sol$weight
    }

    res <- sa_solve(inst_rep, solver)
    if (length(res$edges) == 0) {
        g <- igraph::induced_subgraph(instance, vids = res$vertices)
    } else {
        g <- igraph::subgraph_from_edges(instance, eids = res$edges)
    }
    weight <- get_weight(g)
    res$weight <- weight
    solution(g, weight, solved_to_optimality = FALSE, warm_start_solution = res)
}

test_that("SA solver works on specific test", {
    solver <- annealing_solver(initial_temperature = 10)
    g <- igraph::make_ring(5)
    V(g)$weight <- 1:-3
    E(g)$weight <- 1
    solution <- solve_mwcsp(solver, g)
    expect_equal(solution$weight, 2)
})

test_that("The SA solver does not crush on GAM instances", {
    solver <- annealing_solver()
    solution <- solve_mwcsp(solver, gam_example)
    expect_gte(length(V(solution$graph)), 0)
})

# TODO: decide how to run this test, it is not stable enough to be run on CI
#test_that("The SA solver gives a good solution for a GAM instance", {
#    solver <- annealing_solver(schedule = "boltzmann",
#                               initial_temperature = 2.0, final_temperature = 0.125)
#    solution <- solve_mwcsp(solver, gam_example)
#    expect_gt(solution$weight, 200)
#})

test_that("SA solver handles graphs with self-loops", {
    # Regression test: self-loop edges caused a crash because DynamicGraph
    # ignores self-loops (null token), making vertices appear isolated in the
    # dynamic connectivity structure even though a self-loop edge remained in
    # module_edges. This led to orphan edges and eventual assertion failure.
    solver <- annealing_solver()
    g <- igraph::make_ring(3)
    g <- igraph::add_edges(g, c(1L, 1L))  # self-loop on vertex 1
    V(g)$weight <- c(2, -1, -1)
    E(g)$weight <- c(1, 1, 1, 0.5)
    expect_no_error(solve_mwcsp(solver, g))
})

mwcs_solver_class <- "mwcs_solver"
parameter_class <- "solver_parameter"

EPS <- 1e-8

instance_from_graph <- function(graph) {
    edgelist <- as_edgelist(graph, names = FALSE)
    list(edgelist = matrix(as.integer(edgelist), ncol = 2),
         size = length(V(graph)))
}

attr_sets <- list(V = list(list  = igraph::vertex_attr_names,
                      query = igraph::vertex_attr,
                      name  = "nodes"),
                  E = list(list  = igraph::edge_attr_names,
                      query = igraph::edge_attr,
                      name  = "edges"))

validate_attr_values <- function(graph, attr, type = "V", nonnegative = FALSE) {
    attr_set <- attr_sets[[type]]

    if (!(attr %in% attr_set$list(graph))) {
        return(paste0("Couldn't find ", attr_set$name, " attribute '", attr, "'"))
    }
    values <- as.numeric(attr_set$query(graph, attr))
    if (!all(is.finite(values))) {
        return(paste0("All values of ", attr_set$name, " attribute '", attr,
                        "' must be finite."))
    }
    if (nonnegative) {
        if (any(values < 0)) {
            return(paste0("All values of ", attr_set$name, " attribute '", attr,
                        "' must be nonnegative."))
        }
    }
    NULL
}

check_mwcs_solver <- function(x) {
    if (!inherits(x, mwcs_solver_class)) {
        stop("Not a MWCS solver")
    }
}

check_graph <- function(g) {
    if (!igraph::is_igraph(g)) {
        stop("Not a graph object")
    }
    if (igraph::is_directed(g)) {
        stop("Not an undirected graph")
    }
    if (igraph::any_multiple(g)) {
        stop("Multiple edges (edges connecting the same vertices) are not supported")
    }
}

solver_ctor <- function(classes) {
    params <- as.list(parent.frame(1L))
    x <- structure(list(), class = classes)
    do.call(set_parameters, c(list(solver = x), params))
}

#' Solves a MWCS instance.
#'
#' Generic function for solving MWCS instances using solvers collected in the package.
#'
#' MWCS instance here is represented as an undirected graph, an `igraph` object.
#' The package supports four types of instances: Simple MWCS, Generalized MWCS,
#' Budget MWCS, signal MWCS problems. All the necessary weights and costs are
#' passed by setting vertex and edge attributes. See [get_instance_type] to check
#' if the `igraph` object is a correct MWCS instance. For Simple MWCS problem
#' numeric vertex attribute `weight` must be set. For generalized version `weight`s
#' can be provided for edges. For budget version of the problem in addition to
#' vertex weights it is required that `igraph` object would have `budget_cost` vertex
#' attribute with positive numeric values.
#'
#' Signal MWCS instance is quite different. There is no `weight` attribute for
#' neither vertices nor edges. Instead, vertex and edge attribute `signal` should
#' be provided with signal names. A numeric vector containing weights for the signals
#' should be assigned to graph attribute `signals`.
#'
#' See vignette for description of the supported problems. See `igraph` package
#' documentation for more details about getting/setting necesasry attributes.
#'
#' @order 1
#' @examples
#'
#' library(igraph)
#'
#' # for a MWCS instance
#'
#' data(mwcs_example)
#' head(V(mwcs_example)$weight)
#'
#' # for a GMWCS instance
#' data(gmwcs_example)
#' head(E(gmwcs_example)$weight)
#'
#' # for a SGMWCS instance
#' data(sgmwcs_example)
#' head(V(sgmwcs_example)$signal)
#' head(E(sgmwcs_example)$signal)
#'
#' head(sgmwcs_example$signals)
#'
#' @param solver a solver object returned by rmwcs_solver, annealing_solver, rnc_solver or virgo_solver.
#' @param instance an MWCS instance, an igraph object with problem-related vertex, edge and graph attributes. See details.
#' @param ... other arguments to be passed.
#' @return An object of class `mwcsp_solution` consisting of resulting subgraph,
#' its weight and other information about solution provided.
#' @export
solve_mwcsp <- function(solver, instance, ...) {
    check_mwcs_solver(solver)
    check_graph(instance)
    inst_type <- get_instance_type(instance)
    if (!inst_type$valid) {
        stop(paste("Not a valid instance, call get_instance_type() to see errors"))
    }

    UseMethod("solve_mwcsp")
}

#' Sets values of specific parameters
#' @param solver a solver
#' @param ... listed parameter names and values assigned to them
#' @return The solver with parameters changed.
#' @export
set_parameters <- function(solver, ...) {
    params <- parameters(solver)
    actual <- list(...)
    for (param_name in names(actual)) {
        param <- params[[param_name]]
        if (!param_name %in% names(params)) {
            warning(paste("Unknown parameter", param_name, "for the solver"))
        } else {
            solver[[param_name]] <- check_parameter(param, actual[[param_name]])
        }
    }
    solver
}

#' @export
`[<-.mwcs_solver` <- function(x, i = NULL, j = NULL, value) {
    params <- list(solver = x)
    params[[i]] <- value
    do.call(set_parameters, params)
}

#' @export
`[.mwcs_solver` <- function(x, i = NULL, j = NULL, value) {
    x[[i]]
}

#' @export
print.mwcs_solver <- function(x, ...) {
    check_mwcs_solver(x)
    cat("MWCS Solver: ", class(x)[1], "\n\n")

    cat("Parameters:\n")
    get_param_name <- function(x) setNames(x$name, NULL)
    param_names <- sapply(parameters(x), get_param_name)
    params <- data.frame(name = param_names,
                         value = sapply(param_names,
                            function(y) if(is.null(x[y])) "NULL" else x[y]),
                         row.names = NULL)
    print(params, right = FALSE, row.names = FALSE)
    cat("\n")

    invisible(x)
}

#' @export
parameters.default <- function(...) {
    stop("Object is not an MWCS solver or it is an abstract solver")
}

#' The method returns all parameters supported by specific solver
#' @param solver a solver object
#' @return A table containing parameter names and possible values for each parameter.
#' @export
parameters <- function(solver) UseMethod("parameters")

#' Sets time limitation for a solver
#' @param x a variable name.
#' @param value a value to be assigned to x.
#' @return The solver with new timelimit set.
#' @export
`timelimit<-` <- function(x, value) {
    set_parameters(x, timelimit = value)
}

check_signals <- function(instance) {
    if (!is.numeric(instance$signals) || !all(is.finite(instance$signals))) {
        return("`signals` attribute is not a vector of finite numbers")
    }

    if (any(duplicated(names(instance$signals)))) {
        return("Graph `signals` attribute has duplicated names")
    }

    if (!"signal" %in% vertex_attr_names(instance)) {
        return("No `signal` attribute for nodes")
    }

    if (!all(V(instance)$signal %in% names(instance$signals))) {
        return("All node signals should be present in `signals` graph attribute")
    }

    if (!"signal" %in% edge_attr_names(instance)) {
        return("No `signal` attribute for edges")
    }

    if (!all(E(instance)$signal %in% names(instance$signals))) {
        return("All edge signals should be present in `signals` graph attribute")
    }
    NULL
}

#' Check the type and the validity of an MWCS instance
#' @param instance `igraph` object, containing an instance to be checked
#' @return A list with members `type` containing the type of the instance,
#' `valid` -- boolean flag indicating whether the instance is valid or not,
#' `errors` -- a character vector containing the error messages
#' @examples
#' data(mwcs_example)
#' get_instance_type(mwcs_example)
#' @return A list with two fields: the type of the instance with which it will
#' be treated by solve_mwcsp function and boolean showing validness of the instance.
#' @export
get_instance_type <- function(instance) {
    check_graph(instance)
    res <- list(type="unknown", valid=FALSE, errors=NULL)
    if ("signals" %in% names(graph_attr(instance))) {
        res$type <- "SGMWCS"
        res$errors <- check_signals(instance)
    } else if ("weight" %in% edge_attr_names(instance) &&
               "weight" %in% vertex_attr_names(instance)) {

        res$type <- "GMWCS"
        res$errors <- c(validate_attr_values(instance, "weight", "V"),
                        validate_attr_values(instance, "weight", "E"))
    } else if ("weight" %in% vertex_attr_names(instance)) {
        if ("budget_cost" %in% vertex_attr_names(instance)) {
            res$type <- "Budget MWCS"
            res$errors <- validate_attr_values(instance, "budget_cost", "V",
                                               nonnegative = TRUE)
        } else {
            res$type <- "MWCS"
        }
        res$errors <- c(res$errors, validate_attr_values(instance,
                                                         "weight", "V"))
    } else {
        res$errors <- "Can't determine type of the instance"
    }

    res$valid <- is.null(res$errors)

    res
}

library(igraph)

test_that("normalize_sgmwcs_instance works", {
    et <- data.frame(from = c(1, 2, 2),
                     to = c(2, 3, 4),
                     signal = c("S5", "S6", "S7"),
                     weight = c(-1, -1, -1))
    nt <- data.frame(node=c(1, 2, 3, 4),
                     signal=c("S1", "S2", "S3", "S4"),
                     weight = c(-1, -1, -1, -1))
    g <- graph_from_data_frame(et, directed = FALSE, vertices = nt)
    instance <- normalize_sgmwcs_instance(g)
    expect_true("signals" %in% names(graph.attributes(instance)))
    expect_equal(length(instance$signals), 7)

    expect_warning(normalize_sgmwcs_instance(instance))

    # absent attributes
    expect_error(normalize_sgmwcs_instance(g, nodes.weight.column = "score"))
    expect_error(normalize_sgmwcs_instance(g, edges.weight.column = "score"))
    expect_error(normalize_sgmwcs_instance(g, nodes.group.by = "mz"))
    expect_error(normalize_sgmwcs_instance(g, edges.group.by = "mz"))

    et1 <- et
    et1$weight[1] <- Inf
    g1 <- graph_from_data_frame(et1, directed = FALSE, vertices = nt)
    expect_error(normalize_sgmwcs_instance(g1))

    nt1 <- nt
    nt1$weight[1] <- Inf
    g1 <- graph_from_data_frame(et, directed = FALSE, vertices = nt1)
    expect_error(normalize_sgmwcs_instance(g1))

    nt1 <- nt
    nt1$weight <- c(1, 2, 3, 4)
    nt1$signal <- c("S1", "S1", "S3", "S4")
    g1 <- graph_from_data_frame(et, directed = FALSE, vertices = nt1)
    expect_error(normalize_sgmwcs_instance(g1))

})

# METNET-T-57
test_that("normalize_sgmwcs_instance handles NA signals as unique", {
    et <- data.frame(from = c(1, 2, 2),
                     to = c(2, 3, 4),
                     signal = c(NA, NA, NA),
                     weight = c(-1, -2, 3))
    nt <- data.frame(node=c(1, 2, 3, 4),
                     signal=c(NA, NA, NA, "S4"),
                     weight = c(-10, 1, 4, -1))
    g <- graph_from_data_frame(et, directed = FALSE, vertices = nt)
    instance <- normalize_sgmwcs_instance(g)
    expect_true(get_instance_type(instance)$valid)
})

test_that("get_instance_type fails when signal names are repeated", {
    et <- data.frame(from = c(1, 2, 2),
                     to = c(2, 3, 4),
                     signal = c("S5", "S6", "S7"),
                     weight = c(-1, -1, -1))
    nt <- data.frame(node=c(1, 2, 3, 4),
                     signal=c("S1", "S2", "S3", "S4"),
                     weight = c(-1, -1, -1, -1))
    g <- graph_from_data_frame(et, directed = FALSE, vertices = nt)
    instance <- normalize_sgmwcs_instance(g)
    instance$signals <- c(instance$signals, instance$signals[1])
    expect_true(!get_instance_type(instance)$valid)
})

to_signal_instance <- function(instance) {
    signal_instance <- NULL
    inst_type <- get_instance_type(instance)

    if (inst_type$type == "SGMWCS" && inst_type$valid) {
        signal_instance <- instance
    } else if (inst_type$type == "GMWCS" && inst_type$valid) {
        signal_instance <- normalize_sgmwcs_instance(instance,
                                                     nodes.group.by = NULL,
                                                     edges.group.by = NULL)
    } else if (inst_type$type == "MWCS" && inst_type$valid) {
        inst2 <- instance
        E(inst2)$weight <- 0
        signal_instance <- normalize_sgmwcs_instance(inst2,
                                                     nodes.group.by = NULL,
                                                     edges.group.by = NULL)
    } else {
        stop("Not a valid MWCS, GMWCS or SGMWCS instance")
    }
    signal_instance
}


#' Helper function to convert an `igraph` object into a proper SGMWCS instance
#'
#' This function generates new `igraph` object with additional `signals` field added.
#' The way the instance is constructed is defined by the function parameters.
#' Nodes and edges are grouped separately, grouping columns are defined
#' by `nodes.group.by` and `edges.group.by` arguments. `group.only.positive` param specifies
#' whether to group only positive-weighted (specified by `nodes/edges.weight.column`) nodes and edges.
#'
#' @param g Graph to convert
#' @param nodes.weight.column Nodes column name (e.g. weight, score, w) for scoring
#' @param edges.weight.column Edges column name for scoring
#' @param nodes.group.by Nodes grouping column (e.g. signal, group, class)
#' @param edges.group.by Edges grouping column
#' @param group.only.positive Whether to group only positive-scored nodes/edges#'
#' @return An `igraph` object with proper attributes set.
#' @export
#' @examples
#' data("gatom_example")
#' normalize_sgmwcs_instance(gatom_example)
#'
#' @importFrom stats setNames
normalize_sgmwcs_instance <- function(g,
                                      nodes.weight.column="weight",
                                      edges.weight.column="weight",
                                      nodes.group.by="signal",
                                      edges.group.by="signal",
                                      group.only.positive=TRUE) {
    check_graph(g)
    if ("signals" %in% names(graph.attributes(g))) {
        warning("Input graph alredy looks like SGMWCS instance, doing nothing")
        return(g)
    }

    nt <- as_data_frame(g, what="vertices")
    if (!nodes.weight.column %in% colnames(nt)) {
        stop(sprintf("No %s node attribute present", nodes.weight.column))
    }

    if (!all(is.finite(nt[[nodes.weight.column]]))) {
        stop(sprintf("Not all node weights are finite numbers"))
    }

    if (!is.null(nodes.group.by)) {
        if (all(nodes.group.by %in% colnames(nt))) {
            signalParts <- nt[, nodes.group.by, drop=F]
            signalParts[is.na(signalParts)] <- paste0("__n", seq_len(sum(is.na(signalParts))))
            nt$signal <- do.call(paste, c(signalParts, sep="\r"))
            if (group.only.positive) {
                nt$signal <- paste(nt$signal, ifelse(nt[[nodes.weight.column]] > 0,
                                                     "",
                                                     seq_len(nrow(nt))), sep="\r")
            }
        } else {
            stop(sprintf("Can't collapse nodes, not all fields are present: %s",
                      paste0(setdiff(nodes.group.by, colnames(nt)), collapse=", ")))
        }

    } else {
        nt$signal <- paste0("sn_", seq_len(nrow(nt)))
    }
    nt <- nt[, c("signal", nodes.weight.column)]
    colnames(nt) <- c("signal", "weight")


    et <- as_data_frame(g, what="edges")
    if (!edges.weight.column %in% colnames(et)) {
        stop(sprintf("No %s edge attribute present", edges.weight.column))
    }

    if (!all(is.finite(et[[edges.weight.column]]))) {
        stop(sprintf("Not all edge weights are finite numbers"))
    }

    if (!is.null(edges.group.by)) {
        if (all(edges.group.by %in% colnames(et))) {
            signalParts <- et[, edges.group.by, drop=F]
            signalParts[is.na(signalParts)] <- paste0("__e", seq_len(sum(is.na(signalParts))))
            et$signal <- do.call(paste, c(signalParts, sep="\r"))

            if (group.only.positive) {
                et$signal <- paste(et$signal, ifelse(et[[edges.weight.column]] > 0,
                                                     "",
                                                     seq_len(nrow(et))), sep="\r")
            }
        } else {
            stop(sprintf("Can't collapse edges, not all fields present: %s",
                      paste0(setdiff(edges.group.by, colnames(et)), collapse=", ")))
        }

    } else {
        et$signal <- paste0("se_", seq_len(nrow(et)))
    }

    et <- et[, c("signal", edges.weight.column)]
    colnames(et) <- c("signal", "weight")

    st <- rbind(nt, et)

    st <- unique(st)
    if (any(duplicated(st$signal))) {
        stop(sprintf("Multiple weights are present for signal %s",
                     st$signal[which(duplicated(st$signal))[1]]))
    }

    # renaming signals
    old_signals <- st$signal
    st$signal <- paste0("s", seq_len(nrow(st)))
    old2new <- setNames(st$signal, old_signals)
    nt$signal <- old2new[nt$signal]
    et$signal <- old2new[et$signal]

    ret <- g
    ret$signals <- setNames(st$weight, st$signal)
    V(ret)$signal <- nt$signal
    E(ret)$signal <- et$signal
    ret
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gatom_example}
\alias{gatom_example}
\title{Example of graph from which an SGMWCS instance can be obtained}
\format{
An object of class \code{igraph} of length 194.
}
\usage{
gatom_example
}
\description{
The graph is based on \code{gatom} package
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annealing.R
\name{annealing_solver}
\alias{annealing_solver}
\title{Construct an annealing solver}
\usage{
annealing_solver(
  schedule = c("fast", "boltzmann"),
  initial_temperature = 1,
  final_temperature = 1e-06,
  verbose = FALSE
)
}
\arguments{
\item{schedule}{boltzmann annealing or fast annealing}

\item{initial_temperature}{initial value for the temperature}

\item{final_temperature}{final value for the temperature}

\item{verbose}{whether be verbose or not}
}
\value{
An object of class \code{mwcs_solver}
}
\description{
Simulated annealing is a heuristic method of solving optimization problems.
Typically, it allows to find some good solution in a short time. This
implementation doesn't compute any upper bound on solution, so there is no
guarantee of optimality of solution provided.
}
\details{
Algorithm maintains connected subgraph staring with empty subgraph.
Each iteration one random action is considered. It may be a removal of a
vertex or an edge which does not separate graph or addition of an extra vertex or
an edge neighboring existing graph. If the subgraph is empty all vertices
are considered as candidates to form a subgaph. After candidate is chosen two
subgraph scores are considered: for a new subgraph and for an old one. Simulated
annealing operates with a notion of temperature. The candidate action is
accepted with probability: p(S'|S) = exp(-E / T), where E is weight difference
between subgraphs and T is current temperature.

Temperature is calculated in each iteration. in \code{mwcsr} there are two
temperature schedules supported. So called Boltmann annealing uses the formula:
T(k) = T0 / (ln(1 + k)), while in case of fast annealing this one is used:
T(k) = T0 / k, where k is iteration number.

To tune the algorithm it is useful to realize how typical changes in the goal
function for single actions are distributed. Calculating the acceptance probabilities
at initial temperature and final temperatures may help to choose schedule and
temperatures.
}
\seealso{
\link{rnc_solver} will probably be a better choice with minimal tuning necessary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gam_example}
\alias{gam_example}
\title{GAM instance for MWCS problem}
\format{
A vector of named vertex-weighted igraph instances
}
\source{
\url{http://dimacs11.zib.de/instances/MWCS-GAM.zip}
}
\usage{
gam_example
}
\description{
A dataset containing some real-world instances appeared in network
enrichment analysis tool Shiny GAM
(\doi{10.1093/nar/gkw266}).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvers.R
\name{timelimit<-}
\alias{timelimit<-}
\title{Sets time limitation for a solver}
\usage{
timelimit(x) <- value
}
\arguments{
\item{x}{a variable name.}

\item{value}{a value to be assigned to x.}
}
\value{
The solver with new timelimit set.
}
\description{
Sets time limitation for a solver
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scipjack.R
\name{scipjack_solver}
\alias{scipjack_solver}
\title{Construct a SCIP-jack solver}
\usage{
scipjack_solver(scipstp_bin, config_file = NULL)
}
\arguments{
\item{scipstp_bin}{path to \verb{scipstp binary}.}

\item{config_file}{scipstp-formatted file. Parameter names and values follow
the SCIP configuration format.}
}
\description{
This solver requires the STP extension of the SCIP-jack solver.
To use this class you first need to download and build \code{SCIP-jack} and
\code{SCIPSTP} application.
}
\details{
You can access solver directly using \code{run_scip} function. See example.
}
\examples{
\dontrun{
data("bionet_example")
scip <- scipjack_solver(scipstp_bin='/path/to/scipoptsuite/build/bin/applications/scipstp')
sol <- solve_mwcsp(scip, bionet_example)
}
}
\references{
Rehfeldt D., Koch T. (2019)
"Combining NP-Hard Reduction Techniques and Strong Heuristics in an Exact Algorithm for the Maximum-Weight Connected Subgraph Problem."
\doi{10.1137/17M1145963}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/virgo.R
\name{virgo_solver}
\alias{virgo_solver}
\title{Construct a virgo solver}
\usage{
virgo_solver(
  cplex_dir,
  threads = parallel::detectCores(),
  timelimit = NULL,
  penalty = 0,
  memory = "2G",
  log = 0,
  cplex_bin = NULL,
  cplex_jar = NULL,
  mst = FALSE,
  dryrun = FALSE,
  jvmargs = NULL
)
}
\arguments{
\item{cplex_dir}{a path to dir containing cplex_bin and cplex_jar,
setting this to NULL sets \code{mst} param to \code{TRUE}}

\item{threads}{number of threads for simultaneous computation}

\item{timelimit}{maximum number of seconds to solve the problem}

\item{penalty}{additional edge penalty for graph edges}

\item{memory}{maximum amount of memory(-Xmx flag)}

\item{log}{verbosity level}

\item{cplex_bin}{a path to cplex binary dir}

\item{cplex_jar}{a path to cplex jar file}

\item{mst}{whether to use approximate MST solver, no CPLEX files required with this parameter
is set to \code{TRUE}}

\item{dryrun}{if set to \code{TRUE} only prints the solver command, without actually running it}

\item{jvmargs}{character vector with additional arguments for Java Virtual Machine}
}
\value{
An object of class \code{mwcs_solver}.
}
\description{
This solver uses reformulation of MWCS problem in terms of mixed integer
programming. The later problem can be efficiently solved with
commercial optimization software. Exact version of solver uses CPLEX and requires
it to be installed. CPLEX 12.7.1 or higher is required.
}
\details{
The solver currently does not support repeated negative signals, i.e. every
negative signal should be present only once among all edges and vertices.

You can access solver directly using \code{run_main} function. See example.
}
\examples{
data("sgmwcs_small_instance")
approx_vs <- virgo_solver(mst=TRUE, threads = 1)
approx_vs$run_main("-h")
sol <- solve_mwcsp(approx_vs, sgmwcs_small_instance)
\dontrun{
vs <- virgo_solver(cplex_dir='/path/to/cplex')
sol <- solve_mwcsp(vs, sgmwcs_example)
}
}
\references{
Loboda A., Artyomov M., and Sergushichev A. (2016)
"Solving generalized maximum-weight connected subgraph problem for network enrichment analysis"
\doi{10.1007/978-3-319-43681-4_17}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmwcs.R
\name{rmwcs_solver}
\alias{rmwcs_solver}
\title{Generate a rmwcs solver}
\usage{
rmwcs_solver(
  timelimit = 1800L,
  max_iterations = 1000L,
  beta_iterations = 5L,
  separation = "strong",
  start_constraints = TRUE,
  pegging = TRUE,
  max_age = 10,
  sep_iterations = 10L,
  sep_iter_freeze = 50L,
  heur_iterations = 10L,
  subgradient = "classic",
  beta = 2,
  verbose = FALSE
)
}
\arguments{
\item{timelimit}{Timelimit in seconds}

\item{max_iterations}{Maximum number of iterations}

\item{beta_iterations}{Number of nonimproving iterations until beta is halved}

\item{separation}{Separation: "strong" or "fast"}

\item{start_constraints}{Whether to add flow-conservation/degree constraints at start}

\item{pegging}{variable fixing}

\item{max_age}{number of iterations in aging procedure for non-violated  cuts}

\item{sep_iterations}{Frequency of separating cuts (in iterations)}

\item{sep_iter_freeze}{Number of iterations when a newly separated cut is anaffected by subgradient algorithm.}

\item{heur_iterations}{Frequency of calling heuristic method (in iterations)}

\item{subgradient}{Subgradient: "classic", "average", "cft"}

\item{beta}{Initial step size of subgradient algorithm}

\item{verbose}{Should the solving progress and stats be printed?}
}
\value{
An object of class \code{mwcs_solver}.
}
\description{
The method is based on relax-and-cut approach and allows to solve
Maximum Weight Subgraph Probleam and its budget and cardinality variants.
By constructing lagrangian
relaxation of MWCS problem necessary graph connectivity constraints are introduced
in the objective function giving upper bound on the weight of the optimal
solution. On the other side, primal heuristic uses individul contribution
of the variables to lagrangian relaxation to find possible solution of the initial
problem. The relaxation is then optimized by using iterative subgradient method.
}
\details{
One iteration of algorithm includes solving lagrangian relaxation and updating
lagrange multipliers. It may also contain cuts (or connectivity constraints) separation process, run of
heuristic method, variable fixing routine. The initial step size for
subgradient method can be passed as \code{beta} argument. If there is no improvement in
upper bound in consequtive \code{beta_iterations} iterations the step size is
halved. There are three possible strategies for updating multipliers. See the references
section for the article where differences are discussed.

Some initial cuts are added at the start of the algorithm if \code{start_constraints}
is set to \code{TRUE}. Other constraints are separated on the fly and are
unaffected in the next \code{sep_iter_freeze} iterations of the subgradient mehod.
Then the corresponding lagrange mutipliers are updated from iteration to iteration.
Aging procedure for cuts is incorporated in the algorithm meaning constraint multipliers
are updated for non-violated cuts for up to \code{max_age} iterations from
the point where a cut was violated last time. There are two separation methods
implemented: fast and strong, where tha latter is supposed to minimize number of
variables used in generated constraint while in the former case there is no need to explore
whole graph to construct a constraint.

A variant of MST approximation of PCSTP is used as Primal Heuristic.
See references for more details.

The frequences
of running separation process and heuristic are specified in
\code{sep_iterations} and \code{heur_iterations}.
}
\references{
Álvarez-Miranda E., Sinnl M. (2017)
"A Relax-and-Cut framework for large-scale maximum weight connected subgraph problems"
\doi{10.1016/j.cor.2017.05.015}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvers.R
\name{parameters}
\alias{parameters}
\title{The method returns all parameters supported by specific solver}
\usage{
parameters(solver)
}
\arguments{
\item{solver}{a solver object}
}
\value{
A table containing parameter names and possible values for each parameter.
}
\description{
The method returns all parameters supported by specific solver
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnc_sgmwcs.R
\name{rnc_solver}
\alias{rnc_solver}
\title{Construct relax-and-cut SGMWCS solver}
\usage{
rnc_solver(
  max_iterations = 1000L,
  beta_iterations = 50L,
  heur_iterations = 10L,
  sep_iterations = 10L,
  verbose = FALSE
)
}
\arguments{
\item{max_iterations}{Maximum number of iterations}

\item{beta_iterations}{Number of nonimproving iterations until beta is halved}

\item{heur_iterations}{Frequency of calling heuristic method (in iterations)}

\item{sep_iterations}{Frequency of separating cuts (in iterations)}

\item{verbose}{Should the solving progress and stats be printed?}
}
\value{
An object of class \code{mwcs_solver}
}
\description{
The solver is based on the same approach as rmwcs solver. Modifications to
the original scheme are introduced to tackle problems arising with introduction
of edge weights and signals. It is recommended to use rmwcs solver to solve MWCS
problems, while due to differences in primal heuristic it may be a good pratice
to run both solvers on the same problem.
}
\seealso{
\link{rmwcs_solver}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvers.R, R/virgo.R, R/rmwcs.R,
%   R/rnc_sgmwcs.R, R/annealing.R, R/scipjack.R
\name{solve_mwcsp}
\alias{solve_mwcsp}
\alias{solve_mwcsp.virgo_solver}
\alias{solve_mwcsp.rmwcs_solver}
\alias{solve_mwcsp.rnc_solver}
\alias{solve_mwcsp.simulated_annealing_solver}
\alias{solve_mwcsp.scipjack_solver}
\title{Solves a MWCS instance.}
\usage{
solve_mwcsp(solver, instance, ...)

\method{solve_mwcsp}{virgo_solver}(solver, instance, ...)

\method{solve_mwcsp}{rmwcs_solver}(solver, instance, max_cardinality = NULL, budget = NULL, ...)

\method{solve_mwcsp}{rnc_solver}(solver, instance, ...)

\method{solve_mwcsp}{simulated_annealing_solver}(solver, instance, warm_start, ...)

\method{solve_mwcsp}{scipjack_solver}(solver, instance, ...)
}
\arguments{
\item{solver}{a solver object returned by rmwcs_solver, annealing_solver, rnc_solver or virgo_solver.}

\item{instance}{an MWCS instance, an igraph object with problem-related vertex, edge and graph attributes. See details.}

\item{...}{other arguments to be passed.}

\item{max_cardinality}{integer maximum number of vertices in solution.}

\item{budget}{numeric maximum budget of solution.}

\item{warm_start}{warm start solution, an object of the class mwcsp_solution.}
}
\value{
An object of class \code{mwcsp_solution} consisting of resulting subgraph,
its weight and other information about solution provided.
}
\description{
Generic function for solving MWCS instances using solvers collected in the package.
}
\details{
MWCS instance here is represented as an undirected graph, an \code{igraph} object.
The package supports four types of instances: Simple MWCS, Generalized MWCS,
Budget MWCS, signal MWCS problems. All the necessary weights and costs are
passed by setting vertex and edge attributes. See \link{get_instance_type} to check
if the \code{igraph} object is a correct MWCS instance. For Simple MWCS problem
numeric vertex attribute \code{weight} must be set. For generalized version \code{weight}s
can be provided for edges. For budget version of the problem in addition to
vertex weights it is required that \code{igraph} object would have \code{budget_cost} vertex
attribute with positive numeric values.

Signal MWCS instance is quite different. There is no \code{weight} attribute for
neither vertices nor edges. Instead, vertex and edge attribute \code{signal} should
be provided with signal names. A numeric vector containing weights for the signals
should be assigned to graph attribute \code{signals}.

See vignette for description of the supported problems. See \code{igraph} package
documentation for more details about getting/setting necesasry attributes.
}
\examples{

library(igraph)

# for a MWCS instance

data(mwcs_example)
head(V(mwcs_example)$weight)

# for a GMWCS instance
data(gmwcs_example)
head(E(gmwcs_example)$weight)

# for a SGMWCS instance
data(sgmwcs_example)
head(V(sgmwcs_example)$signal)
head(E(sgmwcs_example)$signal)

head(sgmwcs_example$signals)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bionet_example}
\alias{bionet_example}
\title{Example MWCS instance obtained from BioNet package tutorial}
\format{
An object of class \code{igraph} of length 2559.
}
\usage{
bionet_example
}
\description{
Example MWCS instance obtained from BioNet package tutorial
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sgmwcs_example}
\alias{sgmwcs_example}
\title{Example SGMWCS instance}
\format{
An object of class \code{igraph} of length 194.
}
\usage{
sgmwcs_example
}
\description{
Instance is based on \code{gatom} package.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gmwcs_small_instance}
\alias{gmwcs_small_instance}
\title{Small example of GMWCS instance for demonstration purposes.}
\format{
An object of class \code{igraph} of length 5.
}
\usage{
gmwcs_small_instance
}
\description{
Small example of GMWCS instance for demonstration purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solution.R
\name{get_weight}
\alias{get_weight}
\title{Calculate weight of the solution. MWCS, GMWCS and SGMWCS instances are supported}
\usage{
get_weight(solution)
}
\arguments{
\item{solution}{Either \code{mwcsp_solution} or `igraph`` object representing the solution}
}
\value{
Weight of the subgraph
}
\description{
Calculate weight of the solution. MWCS, GMWCS and SGMWCS instances are supported
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mwcs_small_instance}
\alias{mwcs_small_instance}
\title{Small example of MWCS instance for demonstration purposes.}
\format{
An object of class \code{igraph} of length 5.
}
\usage{
mwcs_small_instance
}
\description{
Small example of MWCS instance for demonstration purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvers.R
\name{set_parameters}
\alias{set_parameters}
\title{Sets values of specific parameters}
\usage{
set_parameters(solver, ...)
}
\arguments{
\item{solver}{a solver}

\item{...}{listed parameter names and values assigned to them}
}
\value{
The solver with parameters changed.
}
\description{
Sets values of specific parameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signal_solvers.R
\name{normalize_sgmwcs_instance}
\alias{normalize_sgmwcs_instance}
\title{Helper function to convert an \code{igraph} object into a proper SGMWCS instance}
\usage{
normalize_sgmwcs_instance(
  g,
  nodes.weight.column = "weight",
  edges.weight.column = "weight",
  nodes.group.by = "signal",
  edges.group.by = "signal",
  group.only.positive = TRUE
)
}
\arguments{
\item{g}{Graph to convert}

\item{nodes.weight.column}{Nodes column name (e.g. weight, score, w) for scoring}

\item{edges.weight.column}{Edges column name for scoring}

\item{nodes.group.by}{Nodes grouping column (e.g. signal, group, class)}

\item{edges.group.by}{Edges grouping column}

\item{group.only.positive}{Whether to group only positive-scored nodes/edges#'}
}
\value{
An \code{igraph} object with proper attributes set.
}
\description{
This function generates new \code{igraph} object with additional \code{signals} field added.
The way the instance is constructed is defined by the function parameters.
Nodes and edges are grouped separately, grouping columns are defined
by \code{nodes.group.by} and \code{edges.group.by} arguments. \code{group.only.positive} param specifies
whether to group only positive-weighted (specified by \code{nodes/edges.weight.column}) nodes and edges.
}
\examples{
data("gatom_example")
normalize_sgmwcs_instance(gatom_example)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvers.R
\name{get_instance_type}
\alias{get_instance_type}
\title{Check the type and the validity of an MWCS instance}
\usage{
get_instance_type(instance)
}
\arguments{
\item{instance}{\code{igraph} object, containing an instance to be checked}
}
\value{
A list with members \code{type} containing the type of the instance,
\code{valid} -- boolean flag indicating whether the instance is valid or not,
\code{errors} -- a character vector containing the error messages

A list with two fields: the type of the instance with which it will
be treated by solve_mwcsp function and boolean showing validness of the instance.
}
\description{
Check the type and the validity of an MWCS instance
}
\examples{
data(mwcs_example)
get_instance_type(mwcs_example)
}
